from typing import Any

import structlog

from seismos_package.config import SeismosConfig


def get_default_logging_conf(log_level: str, formatter: str, formatter_std_lib: str) -> dict[str, Any]:
    config_instance = SeismosConfig()
    app_name = config_instance.APP_NAME

    formatters = {
        "verbose": {
            "format": "%(asctime)s %(levelname)s %(name)s %(message)s",
        },
        "json_formatter": {
            "()": "seismos_package.logging.formatter.LogFormatter",
        },
        "plain_console": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": structlog.dev.ConsoleRenderer(event_key="message"),
        },
        "plain_console_std_lib": {
            "()": "logging.Formatter",
            "format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s",
        },
        "key_value": {
            "()": structlog.stdlib.ProcessorFormatter,
            "processor": structlog.processors.KeyValueRenderer(
                key_order=["microservice", "timestamp", "level", "event", "logger"]
            ),
        },
    }

    if formatter not in formatters or formatter_std_lib not in formatters:
        raise NotImplementedError("formatter not supported")

    config: dict[str, Any] = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": formatters,
        "handlers": {
            "console": {
                "level": log_level,
                "class": "logging.StreamHandler",
                "formatter": formatter,
            },
            "console_std_lib": {
                "level": log_level,
                "class": "logging.StreamHandler",
                "formatter": formatter_std_lib,
            },
        },
        "loggers": {
            "": {
                "level": log_level,
                "handlers": ["console_std_lib"],
                "propagate": False,
            },
        },
    }

    config["loggers"][app_name] = {
        "level": log_level,
        "handlers": ["console"],
        "propagate": False,
    }

    return config



from seismos_package.logging.configuration import get_default_logging_conf
from seismos_package.logging.logger import LoggingConfigurator

__all__ = ["LoggingConfigurator", "get_default_logging_conf"]



from typing import Optional
import logging
import logging.config
import structlog
from structlog import contextvars
from structlog.typing import EventDict
from seismos_package.logging.configuration import get_default_logging_conf
from structlog.dev import ConsoleRenderer


class LoggingConfigurator:
    def __init__(
        self,
        service_name: str,
        log_level: str = "INFO",
        config: Optional[dict] = None,
        setup_logging_dict: bool = False,
    ):
        self.service_name = service_name
        self.log_level = log_level.upper()
        self.config = config
        self.setup_logging_dict = setup_logging_dict

    @staticmethod
    def add_logger_name(logger: logging.Logger, method_name: str, event_dict: EventDict) -> EventDict:
        """
        Adds the logger name to the log event dictionary, keeping compatibility with structlog's
        processor signature. The `method_name` parameter is included to maintain compatibility with structlog's
        processor signature but is not used directly in this method.
        """

        record = event_dict.get("_record")
        event_dict["name"] = record.name if record else logger.name
        return event_dict

    def get_base_processors(self) -> list:
        """Returns the base processors for structlog, common to all formats."""
        return [
            contextvars.merge_contextvars,
            self.add_logger_name,
            structlog.stdlib.add_log_level,
            structlog.stdlib.filter_by_level,
            structlog.processors.TimeStamper(fmt="iso"),
            structlog.stdlib.PositionalArgumentsFormatter(),
            structlog.processors.StackInfoRenderer(),
            structlog.dev.set_exc_info,
            structlog.processors.format_exc_info,
            structlog.processors.UnicodeDecoder(),
            structlog.processors.ExceptionPrettyPrinter(),
            structlog.stdlib.ExtraAdder(),
            structlog.processors.EventRenamer(to="message")
        ]
    def get_processors(self, formatter: str = "json_formatter") -> list:
        """
        Returns processors based on the formatter type.
        """
        processors = self.get_base_processors()

        if formatter == "plain_console":
            processors.append(ConsoleRenderer(colors=True))
        else:
            processors.append(structlog.stdlib.ProcessorFormatter.wrap_for_formatter)

        return processors
    
    def configure_structlog(
        self,
        custom_processors: Optional[list] = None,
        formatter: str = "json_formatter",
        formatter_std_lib: str = "json_formatter",
    ) -> None:
        """Configures the structlog and standard Python logging."""
        if self.setup_logging_dict:
            logger_init_config = self.config or get_default_logging_conf(
                log_level=self.log_level,
                formatter=formatter,
                formatter_std_lib=formatter_std_lib,
            )

            logging.config.dictConfig(logger_init_config)

        processors = custom_processors or self.get_processors(formatter=formatter)

        structlog.configure(
            processors=processors,
            logger_factory=structlog.stdlib.LoggerFactory(),
            wrapper_class=structlog.stdlib.BoundLogger,
            cache_logger_on_first_use=True,
        )



from datetime import datetime, timezone

from pythonjsonlogger import jsonlogger

from seismos_package.config import config


class LogFormatter(jsonlogger.JsonFormatter):
    def add_fields(self, log_record, record, message_dict):
        super().add_fields(log_record, record, message_dict)
        log_record["microservice"] = config.APP_NAME
        if not log_record.get("timestamp"):
            now = datetime.now(tz=timezone.utc).strftime("%Y-%m-%dT%H:%M:%S.%fZ")
            log_record["timestamp"] = now
        log_record["level"] = log_record.get("level", record.levelname).upper()



import os
from dataclasses import dataclass, field


@dataclass
class SeismosConfig:
    APP_NAME: str = field(init=False)
    ENVIRONMENT: str = field(init=False)
    LOG_LEVEL: str = field(init=False)

    def __post_init__(self):
        self.APP_NAME = os.getenv("APP_NAME", "SeismosApp")
        self.ENVIRONMENT = os.getenv("ENVIRONMENT", "dev")
        self.LOG_LEVEL = os.getenv("LOG_LEVEL", "INFO")


config = SeismosConfig()



import uuid

import structlog
from fastapi import Request
from flask import request as flask_request
from starlette.middleware.base import BaseHTTPMiddleware

logger = structlog.get_logger()


def add_request_context_flask():
    """Middleware for Flask applications."""
    request_id = flask_request.headers.get("X-Request-ID", str(uuid.uuid4()))
    structlog.contextvars.bind_contextvars(
        request_id=request_id, request_method=flask_request.method, request_path=flask_request.path
    )


async def add_request_context_fastapi(request: Request, call_next):
    """Middleware for FastAPI applications."""
    request_id = request.headers.get("x-amzn-trace-id")
    if not request_id:
        request_id = str(uuid.uuid4())

    structlog.contextvars.bind_contextvars(
        request_id=request_id, request_method=request.method, request_path=str(request.url)
    )
    response = await call_next(request)
    structlog.contextvars.clear_contextvars()
    return response


class FastAPIRequestContextMiddleware(BaseHTTPMiddleware):
    """Middleware class for FastAPI using BaseHTTPMiddleware."""

    async def dispatch(self, request: Request, call_next):
        return await add_request_context_fastapi(request, call_next)
